require 'settings'
require 'level'
require 'player'
require 'controllers/controller'
require 'controllers/menu_controller'
require 'items/wall'
require 'items/blowable_wall'
require 'items/beamer'

class EditorController < Controller
  attr_accessor :lost, :game_timer, :players
  
  def initialize(level = nil)
    @game_timer = 0
    @level = level || Level.new("New Level", [['w'] * 19] * 19, [], nil, true)
    @pos = V(9, 9)
    
    @cursor = Rubygame::Surface.new [40, 40]
    @cursor.fill [50, 50, 0]
    @cursor.alpha = 127
    
    @editing_name = false
    
    super()
    
    Rubygame::Music['menu.wav'].volume = Settings.audio.music_volume_menu
  end
  
  def update(dtime)
    @game_timer += dtime
    
    # cursor movement
    unless @direction.nil? || @direction == V(0, 0)
      if @first_movement.nil? || (@first_movement + 500 < @game_timer && (@last_movement.nil? || @last_movement + 50 < @game_timer))
        @first_movement ||= @game_timer
        @last_movement    = @game_timer
        @pos += @direction
        @pos.ensure_range!(V(0, 0), V(@level.columns - 1, @level.rows - 1))
      end
    else
      @last_movement = @first_movement = nil
    end
    
    # set field
    field = @level[@pos]
    unless @creating.nil?
      field.items.clear
      @creating.new(field) unless @creating == :nothing
    end
    
    @level.update(dtime)
  end
  
  def handle_event(event)
    super event
    
    @direction = V(0, 0)
    @direction += V(0, -1) if @keys.include?(:up)
    @direction += V(0, 1)  if @keys.include?(:down)
    @direction += V(-1, 0) if @keys.include?(:left)
    @direction += V(1, 0)  if @keys.include?(:right)
    
    field = @level[@pos]
    unless @editing_name
      @creating = nil
      @creating = Wall         if @keys.include?(:w)
      @creating = BlowableWall if @keys.include?(:e)
      @creating = :nothing     if @keys.include?(:r)
      
      if @keys.include?(:s)
        @level.start_points.push(field.coords) unless @level.start_points.delete(field.coords)
      elsif @keys.include?(:n)
        @editing_name = true
      end
    else
      if event.class == Rubygame::Events::KeyPressed
        if /[\w !\?\-#0-9]/i =~ event.string
          @level.name += event.string
        elsif event.key == :backspace
          @level.name.chop!
        elsif event.key == :return
          @editing_name = false
        end
      end
    end
    
    if @keys.include?(:escape)
      @level.file ||= "levels/level#{Level.count_levels + 1}.yml"
      @level.save
      finish!(MenuController.new)
    end
  end
  
  def draw(srf)
    srf.fill [50, 50, 0]
    offset = V(0, 0)
    
    @level.draw srf, offset
    
    @cursor.blit srf, (@pos * 40).to_a
    
    pos = V(Game.screen.width - 230, 10)
    unless @editing_name
      [
        ["w", "Wall"],
        ["e", "Sand"],
        ["r", "Nothing"],
        ["s", "Start point"],
        ["n", "Change name"]
      ].each_with_index do |strings, i|
        strings.each_with_index do |s, j|
          t = Game.font.render(s, false, [255, 255, 255])
          t.blit srf, (pos + V(j*50, i*30)).to_a
        end
      end
    else
      [
        "Enter name:",
        "#{@level.name}#{(@game_timer/500).floor % 2 == 0 ? "|" : ""}"
      ].each_with_index do |s, i|
        if s.length > 0
          t = Game.font.render(s, false, [255, 255, 255])
          t.blit srf, (pos + V(0, i*30)).to_a
        end
      end
    end
  end
end
