require 'settings'
require 'level'
require 'player'
require 'controllers/controller'
require 'controllers/levelselection_menu_controller'
require 'fadeover_effects/flash_fade_effect'
require 'fadeover_effects/crossfade_effect'

class GameController < Controller
  attr_accessor :lost, :game_timer, :players
  
  def initialize(level, players = [], all_players = [])
    @game_timer = 0
    
    @level = level
    Game.current_level = @level
    
    @all_players = all_players
    @players = players
    @players.each do |player|
      sp = @level.start_points[player.number]
      player.prepare_for_game(sp)
    end
    
    #Rubygame::Sound["cinematic-boom.wav"].play
    Rubygame::Music['menu.wav'].volume = Settings.audio.music_volume_game
    
    super()
  end
  
  def update(dtime)
    @game_timer += dtime
    
    @players.each do |p|
      dt = dtime
      while dt > 0
        p.update([dt, 10].min)
        dt -= 10
      end
    end
    
    @level.update(dtime)
    
    @players.each_with_index do |p, i|
      if p.lives <= 0
        @players.delete(p)
        finish!(LevelselectionMenuController.new(@all_players), FlashFadeEffect.new) if @players.size <= 1
      end
    end
  end
  
  def handle_event(event)
    super event
    
    @players.each { |p| p.handle_event(event) }
    finish!(LevelselectionMenuController.new(@all_players), CrossfadeEffect.new) if @keys.include?(:escape)
  end
  
  def draw(srf)
    srf.fill [50, 50, 0]
    offset = V(0, 0)
    
    @level.draw srf, offset
    
    @players.each do |p|
      p.draw(srf, offset)
      p.draw_stats(srf, offset + V(@level.width, p.number * 180))
    end
  end
end
