class Sprite
  attr_accessor :animations, :current_animation
  
  def initialize(file, animations = {})
    @srf = Rubygame::Surface[file]
    @animations = animations
    @current_frame = 0
  end
  
  def current_animation=(i)
    if i != @current_animation
      @current_animation = i
      @current_frame = 0
    end
  end
  
  def update(dtime)
    return if @current_animation.nil?
    anim = @animations[@current_animation]
    @current_frame += anim[:framerate] * dtime
    @current_frame = 0 if @current_frame.floor >= anim[:frames]
  end
  
  def draw(srf, pos, animation = nil, frame = nil)
    fs = 40
    frame     ||= @current_frame.floor
    animation ||= @current_animation
    
    unless animation.nil?
      y = @animations[animation][:row] * fs
      src = [frame*fs, y, fs, fs]
    else
      src = [0, 0, fs, fs]
    end
    
    @srf.blit srf, pos.to_a, src
  end
end
