class Vector
  attr_accessor :x, :y
  
  def initialize(x, y = nil)
    if y.nil?
      @x = x[0]
      @y = x[1]
    else
      @x = x
      @y = y
    end
  end
  
  def [](i)
    i == 0 ? @x : @y
  end
  def []=(i, val)
    if i == 0
      @x = val
    else
      @y = val
    end
  end
  
  def +(other)
    Vector.new(self) if other.nil?
    Vector.new(@x + other[0], @y + other[1])
  end
  
  def -(other)
    Vector.new(self) if other.nil?
    Vector.new(@x - other[0], @y - other[1])
  end
  
  def *(factor)
    Vector.new(@x * factor, @y * factor)
  end
  def /(divisor)
    Vector.new(@x / divisor, @y / divisor)
  end
  
  def abs
    Math.sqrt(@x*@x + @y*@y)
  end
  
  def normalize
    self * (1/abs)
  end
  def normalize!
    f = 1/abs
    @x *= f
    @y *= f
  end
  
  def floor
    Vector.new(@x.floor, @y.floor)
  end
  def ceil
    Vector.new(@x.ceil, @y.ceil)
  end
  
  def ==(other)
    return false if other.nil?
    @x == other[0] && @y == other[1]
  end
  
  def ensure_range!(min, max)
    @x = [@x, min.x].max
    @y = [@y, min.y].max
    @x = [@x, max.x].min
    @y = [@y, max.y].min
  end
  
  def to_s
    "(#{@x}; #{@y})"
  end
  def to_a
    [@x, @y]
  end
end

def V(x, y = nil)
  Vector.new(x, y)
end
